
/**
 * imageBG
 *  the entire background div. Should have background-image:url(image url here)
 *
 * @param left         the absoloute left offset of the div in pixels
 * @param top          the absoloute position offset of the top in pixels
 * @param height      the height of the image (and div) in pixels
 * @param width       the width of the image (and div) in pixels
 * @param img_id      the id attribute of the div
 */
 function imageBG(top, left, height, width, img_id) {
	left = findInt(left);
	top = findInt(top);
	height = findInt(height);
	width = findInt(width);
  
	this.id = img_id;

  //The Rect that represents the whole div
  //Set the points absolutely relative to the page  
	this.imageBox = new Rect();
  this.SetOffset(top, left, height, width);
    
  this.canvases = new Array();              //Each canvas represents one shape
  this.counter = 1;                                //Count of the canvases 
  this.currently_active = 0;  //Nothing currently active	
  
}

imageBG.prototype.ChangeCanvas = function(container_id) {
	var num = findInt(container_id);
	this.canvases[num].ChangeMe( );	
};
imageBG.prototype.SetOffset = function(top, left, height, width) {
  left = findInt(left);
  top= findInt(top);
  height = findInt(height);
  width = findInt(width);
  
	this.imageBox.SetPoints(left, left + width, top, top + height);
	this.maxW = this.imageBox.GetWidth()/2;                                           //max width
	this.maxB = this.imageBox.GetBottomY() - this.imageBox.GetTopY(); //max down
	this.maxR = this.imageBox.GetRightX() - this.imageBox.GetLeftX();    //max right
	this.maxH= this.imageBox.GetHeight();                                              //max height
	this.minL= this.imageBox.GetLeftX();                                                  //min left
};

imageBG.prototype.DisableResize= function ( num_or_id ) {
		var type = "resizable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Disable(type, num);
};
imageBG.prototype.DisableDrag= function ( num_or_id ) {
		var type = "draggable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Disable(type, num);
};

imageBG.prototype.EnableDrag= function ( num_or_id ) {
		var type = "draggable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Enable(type, num);
};
imageBG.prototype.EnableDrag= function ( num_or_id ) {
		var type = "resizable";
		var num = -1;
	if (num_or_id) {
		var num = parseInt(num_or_id);
	}
	this.Enable(type, num);
};

imageBG.prototype.Disable = function ( type, num ) {
	var base = bases.GetContainer();
	if ( num >= 0 ) {
		$ui("#"+base+num).disable(type);		
	} else {
		for ( num = 1; num < this.canvases.length; num++ ) {
			$ui("#"+base+num).disable();		
		}
	}
};
imageBG.prototype.Enable = function ( type, num ) {
	var base = bases.GetContainer();
	if ( num >= 0 ) {
		$ui("#"+base+num).enable(type);		
	} else {
		for ( num = 1; num < this.canvases.length; num++ ) {
			$ui.enable(base+num, type);		
		}
	}
};
imageBG.prototype.Freeze = function ( ) {
	var base = bases.GetContainer();
  var num;
  for ( num = 1; num < this.canvases.length; num++ ) {
    var t = $ui("#"+this.canvases[num].canvas_id);
    t.draggableDisable( );		
    //    t.droppableDisable( );
    t.resizableDisable( );
  }
};
imageBG.prototype.Defrost = function ( ) {
	var base = bases.GetContainer();
  var num;
  for ( num = 1; num < this.canvases.length; num++ ) {
    var s = this.canvases[num].canvas_id;
    $ui("#"+s).enable();		
  }
};

imageBG.prototype.toPrintableString = function( ) {
	var ans = "@";
	var i;
	for ( i= 1; i<this.canvases.length;i++){
		ans += this.canvases[i].toPrintableString() + '@';
	}
	return ans;
};


  
  
imageBG.prototype.AddCanvas = function( type, left, right, top, bottom, new_color ) {

	if ( !new_color ) { new_color = 'red'; };
	
	var num = this.counter++;
	var new_container = bases.GetContainer() + num;
	var new_canvas = bases.GetCanvas() + num;

	var copyMe = $ui('#' + bases.GetContainer()).clone(true).attr("id",new_container).removeClass("hidden");
  copyMe.find("canvas").attr("id", new_canvas).end();
  copyMe.appendTo($ui("#"+this.id));
  $ui("<option>" + ShapeType.Names[type] + " " + num + "</option>").val(new_canvas).appendTo("#which-one");

	this.canvases[num] = new ShapeCanvas(new_color, type, new_container, new_canvas, left, right, top, bottom);
	var parent = this;

	$ui('#'+new_container).resizable({
		name: new_container,
		minWidth: 25,
		minHeight: 25,
		maxWidth: parent.maxWidth,
		maxHeight: parent.maxH,
		handles: {
			e: '.resizeE',
			s: '.resizeS',
			se: '.resizeSE'
		},
		stop: function() {
			parent.ChangeCanvas(new_container); 		
		}
	});
	
  $ui('#'+new_container).draggable({
    containment: 'parent',
    cursor: 'move',
    handle: $ui("#"+bases.GetCanvas()+num),
 		dragPrevention: '.resizeHandler',
    opacity: .7,
    stop: function() {
      parent.ChangeCanvas(new_container); 
    }
  });
  
  $ui("#"+new_container).bind('mouseover', {
  bg_id : parent.id,
  container_id : new_container
  }, activateCanvas);
	
	return;

}




//imageBG.prototype.makeResizable = function( num ) {};
imageBG.prototype.ChangeCanvasColor = function(id_or_num, new_color) {
	var num = findInt(id_or_num);
	this.canvases[num].SetColor(new_color);
};
imageBG.prototype.Draw = function( id_or_num ) {		// optional parameter
	if (id_or_num) {
		var num = parseInt(id_or_num);
		this.canvases[num].Draw();
	} else {
		for ( num = 1; num < this.canvases.length; num++ ) {
			this.canvases[num].Draw();
		}
	}
};
imageBG.prototype.toJSON = function( image_uri ) {
	var image = $ui("#"+this.id);
	var image_name = image_uri.substring(image_uri.lastIndexOf('/')+1);
	var i = image_name.lastIndexOf('.');
	var imagetype = image_name.substring(i+1);
	var all = new Array();

	for(i = 1; i < this.canvases.length; i++) {
    all[i] = {
      "Shape": this.canvases[i].toJSON()
    };
	}
	
	var ans = {
		"image_with_drawings" : {
			"image" : image_name,
			"type" : imagetype,
			"drawings" : all
		}
	};
	return ans;
};
/*imageBG.prototype.HighlightCanvas = function (id_or_num) {
	var num = parseInt(id_or_num);
	this.canvases[num].addClass("highlighted");
};
imageBG.prototype.ClearHighlight = function (id_or_num) {
	var num = parseInt(id_or_num);
	this.canvases[num].removeClass("highlighted");
};
/*imageBG.prototype.Rotate= function ( num_or_id ) {
	var num = parseInt(num_or_id);
	this.canvases[num].Rotate( 45 );
	this.canvases[num].Draw();
	return;
};*/

var activateCanvas = function( event ) {
		$ui(this).find(".resizeHandler").show();
		var m = $ui("#"+event.data.bg_id).one('click', function(){
			$ui("#"+event.data.container_id).find(".resizeHandler").each(function( ){
        $ui(this).hide();
      });
		});
};


function StringIDBasics(resize, canvas) {
	this.resize = resize;
	this.canvas = canvas;
}
StringIDBasics.prototype.GetContainer = function() { return this.resize; };
StringIDBasics.prototype.GetCanvas = function() { return this.canvas; };

var bases = new StringIDBasics("resizeMe_", "canvas_");

